"""
Python module generated from Java source file org.joml.Matrix3f

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Externalizable
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from java.text import DecimalFormat
from java.text import NumberFormat
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Matrix3f(Externalizable, Cloneable, Matrix3fc):
    """
    Contains the definition of a 3x3 matrix of floats, and associated functions to transform
    it. The matrix is column-major to match OpenGL's interpretation, and it looks like this:
    
         m00  m10  m20
         m01  m11  m21
         m02  m12  m22

    Author(s)
    - Kai Burjack
    """

    def __init__(self):
        """
        Create a new Matrix3f and set it to .identity() identity.
        """
        ...


    def __init__(self, mat: "Matrix2fc"):
        """
        Create a new Matrix3f by setting its uppper left 2x2 submatrix to the values of the given Matrix2fc
        and the rest to identity.

        Arguments
        - mat: the Matrix2fc
        """
        ...


    def __init__(self, mat: "Matrix3fc"):
        """
        Create a new Matrix3f and make it a copy of the given matrix.

        Arguments
        - mat: the Matrix3fc to copy the values from
        """
        ...


    def __init__(self, mat: "Matrix4fc"):
        """
        Create a new Matrix3f and make it a copy of the upper left 3x3 of the given Matrix4fc.

        Arguments
        - mat: the Matrix4fc to copy the values from
        """
        ...


    def __init__(self, m00: float, m01: float, m02: float, m10: float, m11: float, m12: float, m20: float, m21: float, m22: float):
        """
        Create a new 3x3 matrix using the supplied float values. The order of the parameter is column-major, 
        so the first three parameters specify the three elements of the first column.

        Arguments
        - m00: the value of m00
        - m01: the value of m01
        - m02: the value of m02
        - m10: the value of m10
        - m11: the value of m11
        - m12: the value of m12
        - m20: the value of m20
        - m21: the value of m21
        - m22: the value of m22
        """
        ...


    def __init__(self, buffer: "FloatBuffer"):
        """
        Create a new Matrix3f by reading its 9 float components from the given FloatBuffer
        at the buffer's current position.
        
        That FloatBuffer is expected to hold the values in column-major order.
        
        The buffer's position will not be changed by this method.

        Arguments
        - buffer: the FloatBuffer to read the matrix values from
        """
        ...


    def __init__(self, col0: "Vector3fc", col1: "Vector3fc", col2: "Vector3fc"):
        """
        Create a new Matrix3f and initialize its three columns using the supplied vectors.

        Arguments
        - col0: the first column
        - col1: the second column
        - col2: the third column
        """
        ...


    def m00(self) -> float:
        ...


    def m01(self) -> float:
        ...


    def m02(self) -> float:
        ...


    def m10(self) -> float:
        ...


    def m11(self) -> float:
        ...


    def m12(self) -> float:
        ...


    def m20(self) -> float:
        ...


    def m21(self) -> float:
        ...


    def m22(self) -> float:
        ...


    def m00(self, m00: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 0 and row 0.

        Arguments
        - m00: the new value

        Returns
        - this
        """
        ...


    def m01(self, m01: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 0 and row 1.

        Arguments
        - m01: the new value

        Returns
        - this
        """
        ...


    def m02(self, m02: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 0 and row 2.

        Arguments
        - m02: the new value

        Returns
        - this
        """
        ...


    def m10(self, m10: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 1 and row 0.

        Arguments
        - m10: the new value

        Returns
        - this
        """
        ...


    def m11(self, m11: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 1 and row 1.

        Arguments
        - m11: the new value

        Returns
        - this
        """
        ...


    def m12(self, m12: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 1 and row 2.

        Arguments
        - m12: the new value

        Returns
        - this
        """
        ...


    def m20(self, m20: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 2 and row 0.

        Arguments
        - m20: the new value

        Returns
        - this
        """
        ...


    def m21(self, m21: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 2 and row 1.

        Arguments
        - m21: the new value

        Returns
        - this
        """
        ...


    def m22(self, m22: float) -> "Matrix3f":
        """
        Set the value of the matrix element at column 2 and row 2.

        Arguments
        - m22: the new value

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix3fc") -> "Matrix3f":
        """
        Set the elements of this matrix to the ones in `m`.

        Arguments
        - m: the matrix to copy the elements from

        Returns
        - this
        """
        ...


    def setTransposed(self, m: "Matrix3fc") -> "Matrix3f":
        """
        Store the values of the transpose of the given matrix `m` into `this` matrix.

        Arguments
        - m: the matrix to copy the transposed values from

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix4x3fc") -> "Matrix3f":
        """
        Set the elements of this matrix to the left 3x3 submatrix of `m`.

        Arguments
        - m: the matrix to copy the elements from

        Returns
        - this
        """
        ...


    def set(self, mat: "Matrix4fc") -> "Matrix3f":
        """
        Set the elements of this matrix to the upper left 3x3 of the given Matrix4fc.

        Arguments
        - mat: the Matrix4fc to copy the values from

        Returns
        - this
        """
        ...


    def set(self, mat: "Matrix2fc") -> "Matrix3f":
        """
        Set the upper left 2x2 submatrix of this Matrix3f to the given Matrix2fc
        and the rest to identity.

        Arguments
        - mat: the Matrix2fc

        Returns
        - this

        See
        - .Matrix3f(Matrix2fc)
        """
        ...


    def set(self, axisAngle: "AxisAngle4f") -> "Matrix3f":
        """
        Set this matrix to be equivalent to the rotation specified by the given AxisAngle4f.

        Arguments
        - axisAngle: the AxisAngle4f

        Returns
        - this
        """
        ...


    def set(self, axisAngle: "AxisAngle4d") -> "Matrix3f":
        """
        Set this matrix to be equivalent to the rotation specified by the given AxisAngle4d.

        Arguments
        - axisAngle: the AxisAngle4d

        Returns
        - this
        """
        ...


    def set(self, q: "Quaternionfc") -> "Matrix3f":
        """
        Set this matrix to be equivalent to the rotation - and possibly scaling - specified by the given Quaternionfc.
        
        This method is equivalent to calling: `rotation(q)`
        
        Reference: <a href="http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToMatrix/">http://www.euclideanspace.com/</a>

        Arguments
        - q: the Quaternionfc

        Returns
        - this

        See
        - .rotation(Quaternionfc)
        """
        ...


    def set(self, q: "Quaterniondc") -> "Matrix3f":
        """
        Set this matrix to a rotation - and possibly scaling - equivalent to the given quaternion.
        
        Reference: <a href="http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToMatrix/">http://www.euclideanspace.com/</a>

        Arguments
        - q: the quaternion

        Returns
        - this
        """
        ...


    def mul(self, right: "Matrix3fc") -> "Matrix3f":
        """
        Multiply this matrix by the supplied `right` matrix.
        
        If `M` is `this` matrix and `R` the `right` matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        transformation of the right matrix will be applied first!

        Arguments
        - right: the right operand of the matrix multiplication

        Returns
        - this
        """
        ...


    def mul(self, right: "Matrix3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def mulLocal(self, left: "Matrix3fc") -> "Matrix3f":
        """
        Pre-multiply this matrix by the supplied `left` matrix and store the result in `this`.
        
        If `M` is `this` matrix and `L` the `left` matrix,
        then the new matrix will be `L * M`. So when transforming a
        vector `v` with the new matrix by using `L * M * v`, the
        transformation of `this` matrix will be applied first!

        Arguments
        - left: the left operand of the matrix multiplication

        Returns
        - this
        """
        ...


    def mulLocal(self, left: "Matrix3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def set(self, m00: float, m01: float, m02: float, m10: float, m11: float, m12: float, m20: float, m21: float, m22: float) -> "Matrix3f":
        """
        Set the values within this matrix to the supplied float values. The result looks like this:
        
        m00, m10, m20
        m01, m11, m21
        m02, m12, m22

        Arguments
        - m00: the new value of m00
        - m01: the new value of m01
        - m02: the new value of m02
        - m10: the new value of m10
        - m11: the new value of m11
        - m12: the new value of m12
        - m20: the new value of m20
        - m21: the new value of m21
        - m22: the new value of m22

        Returns
        - this
        """
        ...


    def set(self, m: list[float]) -> "Matrix3f":
        """
        Set the values in this matrix based on the supplied float array. The result looks like this:
        
        0, 3, 6
        1, 4, 7
        2, 5, 8
        
        This method only uses the first 9 values, all others are ignored.

        Arguments
        - m: the array to read the matrix values from

        Returns
        - this
        """
        ...


    def set(self, m: list[float], off: int) -> "Matrix3f":
        """
        Set the values in this matrix based on the supplied float array in column-major order. The result looks like this:
        
        0, 3, 6
        1, 4, 7
        2, 5, 8
        
        This method only uses the 9 values starting at the given offset.

        Arguments
        - m: the array to read the matrix values from
        - off: the offset into the array

        Returns
        - this
        """
        ...


    def set(self, col0: "Vector3fc", col1: "Vector3fc", col2: "Vector3fc") -> "Matrix3f":
        """
        Set the three columns of this matrix to the supplied vectors, respectively.

        Arguments
        - col0: the first column
        - col1: the second column
        - col2: the third column

        Returns
        - this
        """
        ...


    def determinant(self) -> float:
        ...


    def invert(self) -> "Matrix3f":
        """
        Invert this matrix.

        Returns
        - this
        """
        ...


    def invert(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def transpose(self) -> "Matrix3f":
        """
        Transpose this matrix.

        Returns
        - this
        """
        ...


    def transpose(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def toString(self) -> str:
        """
        Return a string representation of this matrix.
        
        This method creates a new DecimalFormat on every invocation with the format string "`0.000E0;-`".

        Returns
        - the string representation
        """
        ...


    def toString(self, formatter: "NumberFormat") -> str:
        """
        Return a string representation of this matrix by formatting the matrix elements with the given NumberFormat.

        Arguments
        - formatter: the NumberFormat used to format the matrix values with

        Returns
        - the string representation
        """
        ...


    def get(self, dest: "Matrix3f") -> "Matrix3f":
        """
        Get the current values of `this` matrix and store them into
        `dest`.
        
        This is the reverse method of .set(Matrix3fc) and allows to obtain
        intermediate calculation results when chaining multiple transformations.

        Arguments
        - dest: the destination matrix

        Returns
        - the passed in destination

        See
        - .set(Matrix3fc)
        """
        ...


    def get(self, dest: "Matrix4f") -> "Matrix4f":
        ...


    def getRotation(self, dest: "AxisAngle4f") -> "AxisAngle4f":
        ...


    def getUnnormalizedRotation(self, dest: "Quaternionf") -> "Quaternionf":
        ...


    def getNormalizedRotation(self, dest: "Quaternionf") -> "Quaternionf":
        ...


    def getUnnormalizedRotation(self, dest: "Quaterniond") -> "Quaterniond":
        ...


    def getNormalizedRotation(self, dest: "Quaterniond") -> "Quaterniond":
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get3x4(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get3x4(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get3x4(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get3x4(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposed(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def getTransposed(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def getTransposed(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposed(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getToAddress(self, address: int) -> "Matrix3fc":
        ...


    def getTransposedToAddress(self, address: int) -> "Matrix3fc":
        ...


    def get(self, arr: list[float], offset: int) -> list[float]:
        ...


    def get(self, arr: list[float]) -> list[float]:
        ...


    def set(self, buffer: "FloatBuffer") -> "Matrix3f":
        """
        Set the values of this matrix by reading 9 float values from the given FloatBuffer in column-major order,
        starting at its current position.
        
        The FloatBuffer is expected to contain the values in column-major order.
        
        The position of the FloatBuffer will not be changed by this method.

        Arguments
        - buffer: the FloatBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, buffer: "ByteBuffer") -> "Matrix3f":
        """
        Set the values of this matrix by reading 9 float values from the given ByteBuffer in column-major order,
        starting at its current position.
        
        The ByteBuffer is expected to contain the values in column-major order.
        
        The position of the ByteBuffer will not be changed by this method.

        Arguments
        - buffer: the ByteBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, index: int, buffer: "FloatBuffer") -> "Matrix3f":
        """
        Set the values of this matrix by reading 9 float values from the given FloatBuffer in column-major order,
        starting at the specified absolute buffer position/index.
        
        The FloatBuffer is expected to contain the values in column-major order.
        
        The position of the FloatBuffer will not be changed by this method.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: the FloatBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, index: int, buffer: "ByteBuffer") -> "Matrix3f":
        """
        Set the values of this matrix by reading 9 float values from the given ByteBuffer in column-major order,
        starting at the specified absolute buffer position/index.
        
        The ByteBuffer is expected to contain the values in column-major order.
        
        The position of the ByteBuffer will not be changed by this method.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: the ByteBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def setFromAddress(self, address: int) -> "Matrix3f":
        """
        Set the values of this matrix by reading 9 float values from off-heap memory in column-major order,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def setTransposedFromAddress(self, address: int) -> "Matrix3f":
        """
        Set the values of this matrix by reading 9 float values from off-heap memory in row-major order,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the matrix values from in row-major order

        Returns
        - this
        """
        ...


    def zero(self) -> "Matrix3f":
        """
        Set all values within this matrix to zero.

        Returns
        - this
        """
        ...


    def identity(self) -> "Matrix3f":
        """
        Set this matrix to the identity.

        Returns
        - this
        """
        ...


    def scale(self, xyz: "Vector3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def scale(self, xyz: "Vector3fc") -> "Matrix3f":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xyz.x`,
        `xyz.y` and `xyz.z` factors, respectively.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the
        scaling will be applied first!

        Arguments
        - xyz: the factors of the x, y and z component, respectively

        Returns
        - this
        """
        ...


    def scale(self, x: float, y: float, z: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def scale(self, x: float, y: float, z: float) -> "Matrix3f":
        """
        Apply scaling to this matrix by scaling the base axes by the given x,
        y and z factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - z: the factor of the z component

        Returns
        - this
        """
        ...


    def scale(self, xyz: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def scale(self, xyz: float) -> "Matrix3f":
        """
        Apply scaling to this matrix by uniformly scaling all base axes by the given `xyz` factor.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - xyz: the factor for all components

        Returns
        - this

        See
        - .scale(float, float, float)
        """
        ...


    def scaleLocal(self, x: float, y: float, z: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def scaleLocal(self, x: float, y: float, z: float) -> "Matrix3f":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given x,
        y and z factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - z: the factor of the z component

        Returns
        - this
        """
        ...


    def scaling(self, factor: float) -> "Matrix3f":
        """
        Set this matrix to be a simple scale matrix, which scales all axes uniformly by the given factor.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional scaling.
        
        In order to post-multiply a scaling transformation directly to a
        matrix, use .scale(float) scale() instead.

        Arguments
        - factor: the scale factor in x, y and z

        Returns
        - this

        See
        - .scale(float)
        """
        ...


    def scaling(self, x: float, y: float, z: float) -> "Matrix3f":
        """
        Set this matrix to be a simple scale matrix.

        Arguments
        - x: the scale in x
        - y: the scale in y
        - z: the scale in z

        Returns
        - this
        """
        ...


    def scaling(self, xyz: "Vector3fc") -> "Matrix3f":
        """
        Set this matrix to be a simple scale matrix which scales the base axes by `xyz.x`, `xyz.y` and `xyz.z` respectively.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional scaling.
        
        In order to post-multiply a scaling transformation directly to a
        matrix use .scale(Vector3fc) scale() instead.

        Arguments
        - xyz: the scale in x, y and z respectively

        Returns
        - this

        See
        - .scale(Vector3fc)
        """
        ...


    def rotation(self, angle: float, axis: "Vector3fc") -> "Matrix3f":
        """
        Set this matrix to a rotation matrix which rotates the given radians about a given axis.
        
        The axis described by the `axis` vector needs to be a unit vector.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional rotation.
        
        In order to post-multiply a rotation transformation directly to a
        matrix, use .rotate(float, Vector3fc) rotate() instead.

        Arguments
        - angle: the angle in radians
        - axis: the axis to rotate about (needs to be Vector3f.normalize() normalized)

        Returns
        - this

        See
        - .rotate(float, Vector3fc)
        """
        ...


    def rotation(self, axisAngle: "AxisAngle4f") -> "Matrix3f":
        """
        Set this matrix to a rotation transformation using the given AxisAngle4f.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional rotation.
        
        In order to apply the rotation transformation to an existing transformation,
        use .rotate(AxisAngle4f) rotate() instead.
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - axisAngle: the AxisAngle4f (needs to be AxisAngle4f.normalize() normalized)

        Returns
        - this

        See
        - .rotate(AxisAngle4f)
        """
        ...


    def rotation(self, angle: float, x: float, y: float, z: float) -> "Matrix3f":
        """
        Set this matrix to a rotation matrix which rotates the given radians about a given axis.
        
        The axis described by the three components needs to be a unit vector.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional rotation.
        
        In order to apply the rotation transformation to an existing transformation,
        use .rotate(float, float, float, float) rotate() instead.
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - angle: the angle in radians
        - x: the x-component of the rotation axis
        - y: the y-component of the rotation axis
        - z: the z-component of the rotation axis

        Returns
        - this

        See
        - .rotate(float, float, float, float)
        """
        ...


    def rotationX(self, ang: float) -> "Matrix3f":
        """
        Set this matrix to a rotation transformation about the X axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotationY(self, ang: float) -> "Matrix3f":
        """
        Set this matrix to a rotation transformation about the Y axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotationZ(self, ang: float) -> "Matrix3f":
        """
        Set this matrix to a rotation transformation about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotationXYZ(self, angleX: float, angleY: float, angleZ: float) -> "Matrix3f":
        """
        Set this matrix to a rotation of `angleX` radians about the X axis, followed by a rotation
        of `angleY` radians about the Y axis and followed by a rotation of `angleZ` radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        This method is equivalent to calling: `rotationX(angleX).rotateY(angleY).rotateZ(angleZ)`

        Arguments
        - angleX: the angle to rotate about X
        - angleY: the angle to rotate about Y
        - angleZ: the angle to rotate about Z

        Returns
        - this
        """
        ...


    def rotationZYX(self, angleZ: float, angleY: float, angleX: float) -> "Matrix3f":
        """
        Set this matrix to a rotation of `angleZ` radians about the Z axis, followed by a rotation
        of `angleY` radians about the Y axis and followed by a rotation of `angleX` radians about the X axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        This method is equivalent to calling: `rotationZ(angleZ).rotateY(angleY).rotateX(angleX)`

        Arguments
        - angleZ: the angle to rotate about Z
        - angleY: the angle to rotate about Y
        - angleX: the angle to rotate about X

        Returns
        - this
        """
        ...


    def rotationYXZ(self, angleY: float, angleX: float, angleZ: float) -> "Matrix3f":
        """
        Set this matrix to a rotation of `angleY` radians about the Y axis, followed by a rotation
        of `angleX` radians about the X axis and followed by a rotation of `angleZ` radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        This method is equivalent to calling: `rotationY(angleY).rotateX(angleX).rotateZ(angleZ)`

        Arguments
        - angleY: the angle to rotate about Y
        - angleX: the angle to rotate about X
        - angleZ: the angle to rotate about Z

        Returns
        - this
        """
        ...


    def rotation(self, quat: "Quaternionfc") -> "Matrix3f":
        """
        Set this matrix to the rotation - and possibly scaling - transformation of the given Quaternionfc.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional rotation.
        
        In order to apply the rotation transformation to an existing transformation,
        use .rotate(Quaternionfc) rotate() instead.
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Quaternion">http://en.wikipedia.org</a>

        Arguments
        - quat: the Quaternionfc

        Returns
        - this

        See
        - .rotate(Quaternionfc)
        """
        ...


    def transform(self, v: "Vector3f") -> "Vector3f":
        ...


    def transform(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def transformTranspose(self, v: "Vector3f") -> "Vector3f":
        ...


    def transformTranspose(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def transformTranspose(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def rotateX(self, ang: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateX(self, ang: float) -> "Matrix3f":
        """
        Apply rotation about the X axis to this matrix by rotating the given amount of radians.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotateY(self, ang: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateY(self, ang: float) -> "Matrix3f":
        """
        Apply rotation about the Y axis to this matrix by rotating the given amount of radians.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotateZ(self, ang: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateZ(self, ang: float) -> "Matrix3f":
        """
        Apply rotation about the Z axis to this matrix by rotating the given amount of radians.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotateXYZ(self, angles: "Vector3f") -> "Matrix3f":
        """
        Apply rotation of `angles.x` radians about the X axis, followed by a rotation of `angles.y` radians about the Y axis and
        followed by a rotation of `angles.z` radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        rotation will be applied first!
        
        This method is equivalent to calling: `rotateX(angles.x).rotateY(angles.y).rotateZ(angles.z)`

        Arguments
        - angles: the Euler angles

        Returns
        - this
        """
        ...


    def rotateXYZ(self, angleX: float, angleY: float, angleZ: float) -> "Matrix3f":
        """
        Apply rotation of `angleX` radians about the X axis, followed by a rotation of `angleY` radians about the Y axis and
        followed by a rotation of `angleZ` radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        rotation will be applied first!
        
        This method is equivalent to calling: `rotateX(angleX).rotateY(angleY).rotateZ(angleZ)`

        Arguments
        - angleX: the angle to rotate about X
        - angleY: the angle to rotate about Y
        - angleZ: the angle to rotate about Z

        Returns
        - this
        """
        ...


    def rotateXYZ(self, angleX: float, angleY: float, angleZ: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateZYX(self, angles: "Vector3f") -> "Matrix3f":
        """
        Apply rotation of `angles.z` radians about the Z axis, followed by a rotation of `angles.y` radians about the Y axis and
        followed by a rotation of `angles.x` radians about the X axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        rotation will be applied first!
        
        This method is equivalent to calling: `rotateZ(angles.z).rotateY(angles.y).rotateX(angles.x)`

        Arguments
        - angles: the Euler angles

        Returns
        - this
        """
        ...


    def rotateZYX(self, angleZ: float, angleY: float, angleX: float) -> "Matrix3f":
        """
        Apply rotation of `angleZ` radians about the Z axis, followed by a rotation of `angleY` radians about the Y axis and
        followed by a rotation of `angleX` radians about the X axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        rotation will be applied first!
        
        This method is equivalent to calling: `rotateZ(angleZ).rotateY(angleY).rotateX(angleX)`

        Arguments
        - angleZ: the angle to rotate about Z
        - angleY: the angle to rotate about Y
        - angleX: the angle to rotate about X

        Returns
        - this
        """
        ...


    def rotateZYX(self, angleZ: float, angleY: float, angleX: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateYXZ(self, angles: "Vector3f") -> "Matrix3f":
        """
        Apply rotation of `angles.y` radians about the Y axis, followed by a rotation of `angles.x` radians about the X axis and
        followed by a rotation of `angles.z` radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        rotation will be applied first!
        
        This method is equivalent to calling: `rotateY(angles.y).rotateX(angles.x).rotateZ(angles.z)`

        Arguments
        - angles: the Euler angles

        Returns
        - this
        """
        ...


    def rotateYXZ(self, angleY: float, angleX: float, angleZ: float) -> "Matrix3f":
        """
        Apply rotation of `angleY` radians about the Y axis, followed by a rotation of `angleX` radians about the X axis and
        followed by a rotation of `angleZ` radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        rotation will be applied first!
        
        This method is equivalent to calling: `rotateY(angleY).rotateX(angleX).rotateZ(angleZ)`

        Arguments
        - angleY: the angle to rotate about Y
        - angleX: the angle to rotate about X
        - angleZ: the angle to rotate about Z

        Returns
        - this
        """
        ...


    def rotateYXZ(self, angleY: float, angleX: float, angleZ: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotate(self, ang: float, x: float, y: float, z: float) -> "Matrix3f":
        """
        Apply rotation to this matrix by rotating the given amount of radians
        about the given axis specified as x, y and z components.
        
        The axis described by the three components needs to be a unit vector.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians
        - x: the x component of the axis
        - y: the y component of the axis
        - z: the z component of the axis

        Returns
        - this
        """
        ...


    def rotate(self, ang: float, x: float, y: float, z: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateLocal(self, ang: float, x: float, y: float, z: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians
        about the specified `(x, y, z)` axis and store the result in `dest`.
        
        The axis described by the three components needs to be a unit vector.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotation(float, float, float, float) rotation().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians
        - x: the x component of the axis
        - y: the y component of the axis
        - z: the z component of the axis
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotation(float, float, float, float)
        """
        ...


    def rotateLocal(self, ang: float, x: float, y: float, z: float) -> "Matrix3f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians
        about the specified `(x, y, z)` axis.
        
        The axis described by the three components needs to be a unit vector.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotation(float, float, float, float) rotation().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians
        - x: the x component of the axis
        - y: the y component of the axis
        - z: the z component of the axis

        Returns
        - this

        See
        - .rotation(float, float, float, float)
        """
        ...


    def rotateLocalX(self, ang: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Pre-multiply a rotation around the X axis to this matrix by rotating the given amount of radians
        about the X axis and store the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotationX(float) rotationX().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate about the X axis
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotationX(float)
        """
        ...


    def rotateLocalX(self, ang: float) -> "Matrix3f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians about the X axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotationX(float) rotationX().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate about the X axis

        Returns
        - this

        See
        - .rotationX(float)
        """
        ...


    def rotateLocalY(self, ang: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Pre-multiply a rotation around the Y axis to this matrix by rotating the given amount of radians
        about the Y axis and store the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotationY(float) rotationY().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate about the Y axis
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotationY(float)
        """
        ...


    def rotateLocalY(self, ang: float) -> "Matrix3f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians about the Y axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotationY(float) rotationY().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate about the Y axis

        Returns
        - this

        See
        - .rotationY(float)
        """
        ...


    def rotateLocalZ(self, ang: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Pre-multiply a rotation around the Z axis to this matrix by rotating the given amount of radians
        about the Z axis and store the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotationZ(float) rotationZ().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate about the Z axis
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotationZ(float)
        """
        ...


    def rotateLocalZ(self, ang: float) -> "Matrix3f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians about the Z axis.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotationZ(float) rotationY().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate about the Z axis

        Returns
        - this

        See
        - .rotationY(float)
        """
        ...


    def rotate(self, quat: "Quaternionfc") -> "Matrix3f":
        """
        Apply the rotation - and possibly scaling - transformation of the given Quaternionfc to this matrix.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `Q` the rotation matrix obtained from the given quaternion,
        then the new matrix will be `M * Q`. So when transforming a
        vector `v` with the new matrix by using `M * Q * v`,
        the quaternion rotation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying,
        use .rotation(Quaternionfc).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Quaternion">http://en.wikipedia.org</a>

        Arguments
        - quat: the Quaternionfc

        Returns
        - this

        See
        - .rotation(Quaternionfc)
        """
        ...


    def rotate(self, quat: "Quaternionfc", dest: "Matrix3f") -> "Matrix3f":
        """
        Apply the rotation - and possibly scaling - transformation of the given Quaternionfc to this matrix and store
        the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `Q` the rotation matrix obtained from the given quaternion,
        then the new matrix will be `M * Q`. So when transforming a
        vector `v` with the new matrix by using `M * Q * v`,
        the quaternion rotation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying,
        use .rotation(Quaternionfc).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Quaternion">http://en.wikipedia.org</a>

        Arguments
        - quat: the Quaternionfc
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotation(Quaternionfc)
        """
        ...


    def rotateLocal(self, quat: "Quaternionfc", dest: "Matrix3f") -> "Matrix3f":
        """
        Pre-multiply the rotation - and possibly scaling - transformation of the given Quaternionfc to this matrix and store
        the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `Q` the rotation matrix obtained from the given quaternion,
        then the new matrix will be `Q * M`. So when transforming a
        vector `v` with the new matrix by using `Q * M * v`,
        the quaternion rotation will be applied last!
        
        In order to set the matrix to a rotation transformation without pre-multiplying,
        use .rotation(Quaternionfc).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Quaternion">http://en.wikipedia.org</a>

        Arguments
        - quat: the Quaternionfc
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotation(Quaternionfc)
        """
        ...


    def rotateLocal(self, quat: "Quaternionfc") -> "Matrix3f":
        """
        Pre-multiply the rotation - and possibly scaling - transformation of the given Quaternionfc to this matrix.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `Q` the rotation matrix obtained from the given quaternion,
        then the new matrix will be `Q * M`. So when transforming a
        vector `v` with the new matrix by using `Q * M * v`,
        the quaternion rotation will be applied last!
        
        In order to set the matrix to a rotation transformation without pre-multiplying,
        use .rotation(Quaternionfc).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Quaternion">http://en.wikipedia.org</a>

        Arguments
        - quat: the Quaternionfc

        Returns
        - this

        See
        - .rotation(Quaternionfc)
        """
        ...


    def rotate(self, axisAngle: "AxisAngle4f") -> "Matrix3f":
        """
        Apply a rotation transformation, rotating about the given AxisAngle4f, to this matrix.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `A` the rotation matrix obtained from the given AxisAngle4f,
        then the new matrix will be `M * A`. So when transforming a
        vector `v` with the new matrix by using `M * A * v`,
        the AxisAngle4f rotation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying,
        use .rotation(AxisAngle4f).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - axisAngle: the AxisAngle4f (needs to be AxisAngle4f.normalize() normalized)

        Returns
        - this

        See
        - .rotation(AxisAngle4f)
        """
        ...


    def rotate(self, axisAngle: "AxisAngle4f", dest: "Matrix3f") -> "Matrix3f":
        """
        Apply a rotation transformation, rotating about the given AxisAngle4f and store the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `A` the rotation matrix obtained from the given AxisAngle4f,
        then the new matrix will be `M * A`. So when transforming a
        vector `v` with the new matrix by using `M * A * v`,
        the AxisAngle4f rotation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying,
        use .rotation(AxisAngle4f).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - axisAngle: the AxisAngle4f (needs to be AxisAngle4f.normalize() normalized)
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotation(AxisAngle4f)
        """
        ...


    def rotate(self, angle: float, axis: "Vector3fc") -> "Matrix3f":
        """
        Apply a rotation transformation, rotating the given radians about the specified axis, to this matrix.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `A` the rotation matrix obtained from the given angle and axis,
        then the new matrix will be `M * A`. So when transforming a
        vector `v` with the new matrix by using `M * A * v`,
        the axis-angle rotation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying,
        use .rotation(float, Vector3fc).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - angle: the angle in radians
        - axis: the rotation axis (needs to be Vector3f.normalize() normalized)

        Returns
        - this

        See
        - .rotation(float, Vector3fc)
        """
        ...


    def rotate(self, angle: float, axis: "Vector3fc", dest: "Matrix3f") -> "Matrix3f":
        """
        Apply a rotation transformation, rotating the given radians about the specified axis and store the result in `dest`.
        
        When used with a right-handed coordinate system, the produced rotation will rotate a vector 
        counter-clockwise around the rotation axis, when viewing along the negative axis direction towards the origin.
        When used with a left-handed coordinate system, the rotation is clockwise.
        
        If `M` is `this` matrix and `A` the rotation matrix obtained from the given angle and axis,
        then the new matrix will be `M * A`. So when transforming a
        vector `v` with the new matrix by using `M * A * v`,
        the axis-angle rotation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying,
        use .rotation(float, Vector3fc).
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - angle: the angle in radians
        - axis: the rotation axis (needs to be Vector3f.normalize() normalized)
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotation(float, Vector3fc)
        """
        ...


    def lookAlong(self, dir: "Vector3fc", up: "Vector3fc") -> "Matrix3f":
        """
        Apply a rotation transformation to this matrix to make `-z` point along `dir`. 
        
        If `M` is `this` matrix and `L` the lookalong rotation matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`, the
        lookalong rotation transformation will be applied first!
        
        In order to set the matrix to a lookalong transformation without post-multiplying it,
        use .setLookAlong(Vector3fc, Vector3fc) setLookAlong().

        Arguments
        - dir: the direction in space to look along
        - up: the direction of 'up'

        Returns
        - this

        See
        - .setLookAlong(Vector3fc, Vector3fc)
        """
        ...


    def lookAlong(self, dir: "Vector3fc", up: "Vector3fc", dest: "Matrix3f") -> "Matrix3f":
        """
        Apply a rotation transformation to this matrix to make `-z` point along `dir`
        and store the result in `dest`. 
        
        If `M` is `this` matrix and `L` the lookalong rotation matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`, the
        lookalong rotation transformation will be applied first!
        
        In order to set the matrix to a lookalong transformation without post-multiplying it,
        use .setLookAlong(Vector3fc, Vector3fc) setLookAlong().

        Arguments
        - dir: the direction in space to look along
        - up: the direction of 'up'
        - dest: will hold the result

        Returns
        - dest

        See
        - .setLookAlong(Vector3fc, Vector3fc)
        """
        ...


    def lookAlong(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Apply a rotation transformation to this matrix to make `-z` point along `dir`
        and store the result in `dest`. 
        
        If `M` is `this` matrix and `L` the lookalong rotation matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`, the
        lookalong rotation transformation will be applied first!
        
        In order to set the matrix to a lookalong transformation without post-multiplying it,
        use .setLookAlong(float, float, float, float, float, float) setLookAlong()

        Arguments
        - dirX: the x-coordinate of the direction to look along
        - dirY: the y-coordinate of the direction to look along
        - dirZ: the z-coordinate of the direction to look along
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector
        - dest: will hold the result

        Returns
        - dest

        See
        - .setLookAlong(float, float, float, float, float, float)
        """
        ...


    def lookAlong(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float) -> "Matrix3f":
        """
        Apply a rotation transformation to this matrix to make `-z` point along `dir`. 
        
        If `M` is `this` matrix and `L` the lookalong rotation matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`, the
        lookalong rotation transformation will be applied first!
        
        In order to set the matrix to a lookalong transformation without post-multiplying it,
        use .setLookAlong(float, float, float, float, float, float) setLookAlong()

        Arguments
        - dirX: the x-coordinate of the direction to look along
        - dirY: the y-coordinate of the direction to look along
        - dirZ: the z-coordinate of the direction to look along
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector

        Returns
        - this

        See
        - .setLookAlong(float, float, float, float, float, float)
        """
        ...


    def setLookAlong(self, dir: "Vector3fc", up: "Vector3fc") -> "Matrix3f":
        """
        Set this matrix to a rotation transformation to make `-z`
        point along `dir`.
        
        In order to apply the lookalong transformation to any previous existing transformation,
        use .lookAlong(Vector3fc, Vector3fc).

        Arguments
        - dir: the direction in space to look along
        - up: the direction of 'up'

        Returns
        - this

        See
        - .lookAlong(Vector3fc, Vector3fc)
        """
        ...


    def setLookAlong(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float) -> "Matrix3f":
        """
        Set this matrix to a rotation transformation to make `-z`
        point along `dir`.
        
        In order to apply the lookalong transformation to any previous existing transformation,
        use .lookAlong(float, float, float, float, float, float) lookAlong()

        Arguments
        - dirX: the x-coordinate of the direction to look along
        - dirY: the y-coordinate of the direction to look along
        - dirZ: the z-coordinate of the direction to look along
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector

        Returns
        - this

        See
        - .lookAlong(float, float, float, float, float, float)
        """
        ...


    def getRow(self, row: int, dest: "Vector3f") -> "Vector3f":
        ...


    def setRow(self, row: int, src: "Vector3fc") -> "Matrix3f":
        """
        Set the row at the given `row` index, starting with `0`.

        Arguments
        - row: the row index in `[0..2]`
        - src: the row components to set

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `row` is not in `[0..2]`
        """
        ...


    def setRow(self, row: int, x: float, y: float, z: float) -> "Matrix3f":
        """
        Set the row at the given `row` index, starting with `0`.

        Arguments
        - row: the row index in `[0..2]`
        - x: the first element in the row
        - y: the second element in the row
        - z: the third element in the row

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `row` is not in `[0..2]`
        """
        ...


    def getColumn(self, column: int, dest: "Vector3f") -> "Vector3f":
        ...


    def setColumn(self, column: int, src: "Vector3fc") -> "Matrix3f":
        """
        Set the column at the given `column` index, starting with `0`.

        Arguments
        - column: the column index in `[0..2]`
        - src: the column components to set

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `column` is not in `[0..2]`
        """
        ...


    def setColumn(self, column: int, x: float, y: float, z: float) -> "Matrix3f":
        """
        Set the column at the given `column` index, starting with `0`.

        Arguments
        - column: the column index in `[0..2]`
        - x: the first element in the column
        - y: the second element in the column
        - z: the third element in the column

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `column` is not in `[0..2]`
        """
        ...


    def get(self, column: int, row: int) -> float:
        ...


    def set(self, column: int, row: int, value: float) -> "Matrix3f":
        """
        Set the matrix element at the given column and row to the specified value.

        Arguments
        - column: the colum index in `[0..2]`
        - row: the row index in `[0..2]`
        - value: the value

        Returns
        - this
        """
        ...


    def getRowColumn(self, row: int, column: int) -> float:
        ...


    def setRowColumn(self, row: int, column: int, value: float) -> "Matrix3f":
        """
        Set the matrix element at the given row and column to the specified value.

        Arguments
        - row: the row index in `[0..2]`
        - column: the colum index in `[0..2]`
        - value: the value

        Returns
        - this
        """
        ...


    def normal(self) -> "Matrix3f":
        """
        Set `this` matrix to its own normal matrix.
        
        The normal matrix of `m` is the transpose of the inverse of `m`.
        
        Please note that, if `this` is an orthogonal matrix or a matrix whose columns are orthogonal vectors, 
        then this method *need not* be invoked, since in that case `this` itself is its normal matrix.
        In this case, use .set(Matrix3fc) to set a given Matrix3f to this matrix.

        Returns
        - this

        See
        - .set(Matrix3fc)
        """
        ...


    def normal(self, dest: "Matrix3f") -> "Matrix3f":
        """
        Compute a normal matrix from `this` matrix and store it into `dest`.
        
        The normal matrix of `m` is the transpose of the inverse of `m`.
        
        Please note that, if `this` is an orthogonal matrix or a matrix whose columns are orthogonal vectors, 
        then this method *need not* be invoked, since in that case `this` itself is its normal matrix.
        In this case, use .set(Matrix3fc) to set a given Matrix3f to this matrix.

        Arguments
        - dest: will hold the result

        Returns
        - dest

        See
        - .set(Matrix3fc)
        """
        ...


    def cofactor(self) -> "Matrix3f":
        """
        Compute the cofactor matrix of `this`.
        
        The cofactor matrix can be used instead of .normal() to transform normals
        when the orientation of the normals with respect to the surface should be preserved.

        Returns
        - this
        """
        ...


    def cofactor(self, dest: "Matrix3f") -> "Matrix3f":
        """
        Compute the cofactor matrix of `this` and store it into `dest`.
        
        The cofactor matrix can be used instead of .normal(Matrix3f) to transform normals
        when the orientation of the normals with respect to the surface should be preserved.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def getScale(self, dest: "Vector3f") -> "Vector3f":
        ...


    def positiveZ(self, dir: "Vector3f") -> "Vector3f":
        ...


    def normalizedPositiveZ(self, dir: "Vector3f") -> "Vector3f":
        ...


    def positiveX(self, dir: "Vector3f") -> "Vector3f":
        ...


    def normalizedPositiveX(self, dir: "Vector3f") -> "Vector3f":
        ...


    def positiveY(self, dir: "Vector3f") -> "Vector3f":
        ...


    def normalizedPositiveY(self, dir: "Vector3f") -> "Vector3f":
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def equals(self, m: "Matrix3fc", delta: float) -> bool:
        ...


    def swap(self, other: "Matrix3f") -> "Matrix3f":
        """
        Exchange the values of `this` matrix with the given `other` matrix.

        Arguments
        - other: the other matrix to exchange the values with

        Returns
        - this
        """
        ...


    def add(self, other: "Matrix3fc") -> "Matrix3f":
        """
        Component-wise add `this` and `other`.

        Arguments
        - other: the other addend

        Returns
        - this
        """
        ...


    def add(self, other: "Matrix3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def sub(self, subtrahend: "Matrix3fc") -> "Matrix3f":
        """
        Component-wise subtract `subtrahend` from `this`.

        Arguments
        - subtrahend: the subtrahend

        Returns
        - this
        """
        ...


    def sub(self, subtrahend: "Matrix3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def mulComponentWise(self, other: "Matrix3fc") -> "Matrix3f":
        """
        Component-wise multiply `this` by `other`.

        Arguments
        - other: the other matrix

        Returns
        - this
        """
        ...


    def mulComponentWise(self, other: "Matrix3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def setSkewSymmetric(self, a: float, b: float, c: float) -> "Matrix3f":
        """
        Set this matrix to a skew-symmetric matrix using the following layout:
        ```
         0,  a, -b
        -a,  0,  c
         b, -c,  0
        ```
        
        Reference: <a href="https://en.wikipedia.org/wiki/Skew-symmetric_matrix">https://en.wikipedia.org</a>

        Arguments
        - a: the value used for the matrix elements m01 and m10
        - b: the value used for the matrix elements m02 and m20
        - c: the value used for the matrix elements m12 and m21

        Returns
        - this
        """
        ...


    def lerp(self, other: "Matrix3fc", t: float) -> "Matrix3f":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `this`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other matrix
        - t: the interpolation factor between 0.0 and 1.0

        Returns
        - this
        """
        ...


    def lerp(self, other: "Matrix3fc", t: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def rotateTowards(self, direction: "Vector3fc", up: "Vector3fc", dest: "Matrix3f") -> "Matrix3f":
        """
        Apply a model transformation to this matrix for a right-handed coordinate system, 
        that aligns the local `+Z` axis with `direction`
        and store the result in `dest`.
        
        If `M` is `this` matrix and `L` the lookat matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`,
        the lookat transformation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying it,
        use .rotationTowards(Vector3fc, Vector3fc) rotationTowards().
        
        This method is equivalent to calling: `mul(new Matrix3f().lookAlong(new Vector3f(dir).negate(), up).invert(), dest)`

        Arguments
        - direction: the direction to rotate towards
        - up: the model's up vector
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotationTowards(Vector3fc, Vector3fc)
        """
        ...


    def rotateTowards(self, direction: "Vector3fc", up: "Vector3fc") -> "Matrix3f":
        """
        Apply a model transformation to this matrix for a right-handed coordinate system, 
        that aligns the local `+Z` axis with `direction`.
        
        If `M` is `this` matrix and `L` the lookat matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`,
        the lookat transformation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying it,
        use .rotationTowards(Vector3fc, Vector3fc) rotationTowards().
        
        This method is equivalent to calling: `mul(new Matrix3f().lookAlong(new Vector3f(dir).negate(), up).invert())`

        Arguments
        - direction: the direction to orient towards
        - up: the up vector

        Returns
        - this

        See
        - .rotationTowards(Vector3fc, Vector3fc)
        """
        ...


    def rotateTowards(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float) -> "Matrix3f":
        """
        Apply a model transformation to this matrix for a right-handed coordinate system, 
        that aligns the local `+Z` axis with `direction`.
        
        If `M` is `this` matrix and `L` the lookat matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`,
        the lookat transformation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying it,
        use .rotationTowards(float, float, float, float, float, float) rotationTowards().
        
        This method is equivalent to calling: `mul(new Matrix3f().lookAlong(-dirX, -dirY, -dirZ, upX, upY, upZ).invert())`

        Arguments
        - dirX: the x-coordinate of the direction to rotate towards
        - dirY: the y-coordinate of the direction to rotate towards
        - dirZ: the z-coordinate of the direction to rotate towards
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector

        Returns
        - this

        See
        - .rotationTowards(float, float, float, float, float, float)
        """
        ...


    def rotateTowards(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Apply a model transformation to this matrix for a right-handed coordinate system, 
        that aligns the local `+Z` axis with `dir`
        and store the result in `dest`.
        
        If `M` is `this` matrix and `L` the lookat matrix,
        then the new matrix will be `M * L`. So when transforming a
        vector `v` with the new matrix by using `M * L * v`,
        the lookat transformation will be applied first!
        
        In order to set the matrix to a rotation transformation without post-multiplying it,
        use .rotationTowards(float, float, float, float, float, float) rotationTowards().
        
        This method is equivalent to calling: `mul(new Matrix3f().lookAlong(-dirX, -dirY, -dirZ, upX, upY, upZ).invert(), dest)`

        Arguments
        - dirX: the x-coordinate of the direction to rotate towards
        - dirY: the y-coordinate of the direction to rotate towards
        - dirZ: the z-coordinate of the direction to rotate towards
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotationTowards(float, float, float, float, float, float)
        """
        ...


    def rotationTowards(self, dir: "Vector3fc", up: "Vector3fc") -> "Matrix3f":
        """
        Set this matrix to a model transformation for a right-handed coordinate system, 
        that aligns the local `-z` axis with `center - eye`.
        
        In order to apply the rotation transformation to a previous existing transformation,
        use .rotateTowards(float, float, float, float, float, float) rotateTowards.
        
        This method is equivalent to calling: `setLookAlong(new Vector3f(dir).negate(), up).invert()`

        Arguments
        - dir: the direction to orient the local -z axis towards
        - up: the up vector

        Returns
        - this

        See
        - .rotateTowards(float, float, float, float, float, float)
        """
        ...


    def rotationTowards(self, dirX: float, dirY: float, dirZ: float, upX: float, upY: float, upZ: float) -> "Matrix3f":
        """
        Set this matrix to a model transformation for a right-handed coordinate system, 
        that aligns the local `-z` axis with `center - eye`.
        
        In order to apply the rotation transformation to a previous existing transformation,
        use .rotateTowards(float, float, float, float, float, float) rotateTowards.
        
        This method is equivalent to calling: `setLookAlong(-dirX, -dirY, -dirZ, upX, upY, upZ).invert()`

        Arguments
        - dirX: the x-coordinate of the direction to rotate towards
        - dirY: the y-coordinate of the direction to rotate towards
        - dirZ: the z-coordinate of the direction to rotate towards
        - upX: the x-coordinate of the up vector
        - upY: the y-coordinate of the up vector
        - upZ: the z-coordinate of the up vector

        Returns
        - this

        See
        - .rotationTowards(float, float, float, float, float, float)
        """
        ...


    def getEulerAnglesZYX(self, dest: "Vector3f") -> "Vector3f":
        ...


    def getEulerAnglesXYZ(self, dest: "Vector3f") -> "Vector3f":
        ...


    def getEulerAnglesYXZ(self, dest: "Vector3f") -> "Vector3f":
        ...


    def obliqueZ(self, a: float, b: float) -> "Matrix3f":
        """
        Apply an oblique projection transformation to this matrix with the given values for `a` and
        `b`.
        
        If `M` is `this` matrix and `O` the oblique transformation matrix,
        then the new matrix will be `M * O`. So when transforming a
        vector `v` with the new matrix by using `M * O * v`, the
        oblique transformation will be applied first!
        
        The oblique transformation is defined as:
        ```
        x' = x + a*z
        y' = y + a*z
        z' = z
        ```
        or in matrix form:
        ```
        1 0 a
        0 1 b
        0 0 1
        ```

        Arguments
        - a: the value for the z factor that applies to x
        - b: the value for the z factor that applies to y

        Returns
        - this
        """
        ...


    def obliqueZ(self, a: float, b: float, dest: "Matrix3f") -> "Matrix3f":
        """
        Apply an oblique projection transformation to this matrix with the given values for `a` and
        `b` and store the result in `dest`.
        
        If `M` is `this` matrix and `O` the oblique transformation matrix,
        then the new matrix will be `M * O`. So when transforming a
        vector `v` with the new matrix by using `M * O * v`, the
        oblique transformation will be applied first!
        
        The oblique transformation is defined as:
        ```
        x' = x + a*z
        y' = y + a*z
        z' = z
        ```
        or in matrix form:
        ```
        1 0 a
        0 1 b
        0 0 1
        ```

        Arguments
        - a: the value for the z factor that applies to x
        - b: the value for the z factor that applies to y
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def reflect(self, nx: float, ny: float, nz: float, dest: "Matrix3f") -> "Matrix3f":
        ...


    def reflect(self, nx: float, ny: float, nz: float) -> "Matrix3f":
        """
        Apply a mirror/reflection transformation to this matrix that reflects through the given plane
        specified via the plane normal.
        
        If `M` is `this` matrix and `R` the reflection matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        reflection will be applied first!

        Arguments
        - nx: the x-coordinate of the plane normal
        - ny: the y-coordinate of the plane normal
        - nz: the z-coordinate of the plane normal

        Returns
        - this
        """
        ...


    def reflect(self, normal: "Vector3fc") -> "Matrix3f":
        """
        Apply a mirror/reflection transformation to this matrix that reflects through the given plane
        specified via the plane normal.
        
        If `M` is `this` matrix and `R` the reflection matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        reflection will be applied first!

        Arguments
        - normal: the plane normal

        Returns
        - this
        """
        ...


    def reflect(self, orientation: "Quaternionfc") -> "Matrix3f":
        """
        Apply a mirror/reflection transformation to this matrix that reflects about a plane
        specified via the plane orientation.
        
        This method can be used to build a reflection transformation based on the orientation of a mirror object in the scene.
        It is assumed that the default mirror plane's normal is `(0, 0, 1)`. So, if the given Quaternionfc is
        the identity (does not apply any additional rotation), the reflection plane will be `z=0`.
        
        If `M` is `this` matrix and `R` the reflection matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        reflection will be applied first!

        Arguments
        - orientation: the plane orientation

        Returns
        - this
        """
        ...


    def reflect(self, orientation: "Quaternionfc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def reflect(self, normal: "Vector3fc", dest: "Matrix3f") -> "Matrix3f":
        ...


    def reflection(self, nx: float, ny: float, nz: float) -> "Matrix3f":
        """
        Set this matrix to a mirror/reflection transformation that reflects through the given plane
        specified via the plane normal.

        Arguments
        - nx: the x-coordinate of the plane normal
        - ny: the y-coordinate of the plane normal
        - nz: the z-coordinate of the plane normal

        Returns
        - this
        """
        ...


    def reflection(self, normal: "Vector3fc") -> "Matrix3f":
        """
        Set this matrix to a mirror/reflection transformation that reflects through the given plane
        specified via the plane normal.

        Arguments
        - normal: the plane normal

        Returns
        - this
        """
        ...


    def reflection(self, orientation: "Quaternionfc") -> "Matrix3f":
        """
        Set this matrix to a mirror/reflection transformation that reflects through a plane
        specified via the plane orientation.
        
        This method can be used to build a reflection transformation based on the orientation of a mirror object in the scene.
        It is assumed that the default mirror plane's normal is `(0, 0, 1)`. So, if the given Quaternionfc is
        the identity (does not apply any additional rotation), the reflection plane will be `z=0`, offset by the given `point`.

        Arguments
        - orientation: the plane orientation

        Returns
        - this
        """
        ...


    def isFinite(self) -> bool:
        ...


    def quadraticFormProduct(self, x: float, y: float, z: float) -> float:
        ...


    def quadraticFormProduct(self, v: "Vector3fc") -> float:
        ...


    def mapXZY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1 0 0
        0 0 1
        0 1 0
        ```

        Returns
        - this
        """
        ...


    def mapXZY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapXZnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1 0  0
        0 0 -1
        0 1  0
        ```

        Returns
        - this
        """
        ...


    def mapXZnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapXnYnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1  0  0
        0 -1  0
        0  0 -1
        ```

        Returns
        - this
        """
        ...


    def mapXnYnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapXnZY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1  0 0
        0  0 1
        0 -1 0
        ```

        Returns
        - this
        """
        ...


    def mapXnZY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapXnZnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1  0  0
        0  0 -1
        0 -1  0
        ```

        Returns
        - this
        """
        ...


    def mapXnZnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYXZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 1 0
        1 0 0
        0 0 1
        ```

        Returns
        - this
        """
        ...


    def mapYXZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYXnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 1  0
        1 0  0
        0 0 -1
        ```

        Returns
        - this
        """
        ...


    def mapYXnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYZX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 0 1
        1 0 0
        0 1 0
        ```

        Returns
        - this
        """
        ...


    def mapYZX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYZnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 0 -1
        1 0  0
        0 1  0
        ```

        Returns
        - this
        """
        ...


    def mapYZnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYnXZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 -1 0
        1  0 0
        0  0 1
        ```

        Returns
        - this
        """
        ...


    def mapYnXZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYnXnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 -1  0
        1  0  0
        0  0 -1
        ```

        Returns
        - this
        """
        ...


    def mapYnXnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYnZX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0  0 1
        1  0 0
        0 -1 0
        ```

        Returns
        - this
        """
        ...


    def mapYnZX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapYnZnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0  0 -1
        1  0  0
        0 -1  0
        ```

        Returns
        - this
        """
        ...


    def mapYnZnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZXY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 1 0
        0 0 1
        1 0 0
        ```

        Returns
        - this
        """
        ...


    def mapZXY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZXnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 1  0
        0 0 -1
        1 0  0
        ```

        Returns
        - this
        """
        ...


    def mapZXnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZYX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 0 1
        0 1 0
        1 0 0
        ```

        Returns
        - this
        """
        ...


    def mapZYX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZYnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 0 -1
        0 1  0
        1 0  0
        ```

        Returns
        - this
        """
        ...


    def mapZYnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZnXY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 -1 0
        0  0 1
        1  0 0
        ```

        Returns
        - this
        """
        ...


    def mapZnXY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZnXnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0 -1  0
        0  0 -1
        1  0  0
        ```

        Returns
        - this
        """
        ...


    def mapZnXnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZnYX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0  0 1
        0 -1 0
        1  0 0
        ```

        Returns
        - this
        """
        ...


    def mapZnYX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapZnYnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        0  0 -1
        0 -1  0
        1  0  0
        ```

        Returns
        - this
        """
        ...


    def mapZnYnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXYnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1 0  0
         0 1  0
         0 0 -1
        ```

        Returns
        - this
        """
        ...


    def mapnXYnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXZY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1 0 0
         0 0 1
         0 1 0
        ```

        Returns
        - this
        """
        ...


    def mapnXZY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXZnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1 0  0
         0 0 -1
         0 1  0
        ```

        Returns
        - this
        """
        ...


    def mapnXZnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXnYZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1  0 0
         0 -1 0
         0  0 1
        ```

        Returns
        - this
        """
        ...


    def mapnXnYZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXnYnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1  0  0
         0 -1  0
         0  0 -1
        ```

        Returns
        - this
        """
        ...


    def mapnXnYnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXnZY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1  0 0
         0  0 1
         0 -1 0
        ```

        Returns
        - this
        """
        ...


    def mapnXnZY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnXnZnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1  0  0
         0  0 -1
         0 -1  0
        ```

        Returns
        - this
        """
        ...


    def mapnXnZnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYXZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 1 0
        -1 0 0
         0 0 1
        ```

        Returns
        - this
        """
        ...


    def mapnYXZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYXnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 1  0
        -1 0  0
         0 0 -1
        ```

        Returns
        - this
        """
        ...


    def mapnYXnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYZX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 0 1
        -1 0 0
         0 1 0
        ```

        Returns
        - this
        """
        ...


    def mapnYZX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYZnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 0 -1
        -1 0  0
         0 1  0
        ```

        Returns
        - this
        """
        ...


    def mapnYZnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYnXZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 -1 0
        -1  0 0
         0  0 1
        ```

        Returns
        - this
        """
        ...


    def mapnYnXZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYnXnZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 -1  0
        -1  0  0
         0  0 -1
        ```

        Returns
        - this
        """
        ...


    def mapnYnXnZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYnZX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0  0 1
        -1  0 0
         0 -1 0
        ```

        Returns
        - this
        """
        ...


    def mapnYnZX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnYnZnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0  0 -1
        -1  0  0
         0 -1  0
        ```

        Returns
        - this
        """
        ...


    def mapnYnZnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZXY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 1 0
         0 0 1
        -1 0 0
        ```

        Returns
        - this
        """
        ...


    def mapnZXY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZXnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 1  0
         0 0 -1
        -1 0  0
        ```

        Returns
        - this
        """
        ...


    def mapnZXnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZYX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 0 1
         0 1 0
        -1 0 0
        ```

        Returns
        - this
        """
        ...


    def mapnZYX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZYnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 0 -1
         0 1  0
        -1 0  0
        ```

        Returns
        - this
        """
        ...


    def mapnZYnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZnXY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 -1 0
         0  0 1
        -1  0 0
        ```

        Returns
        - this
        """
        ...


    def mapnZnXY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZnXnY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0 -1  0
         0  0 -1
        -1  0  0
        ```

        Returns
        - this
        """
        ...


    def mapnZnXnY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZnYX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0  0 1
         0 -1 0
        -1  0 0
        ```

        Returns
        - this
        """
        ...


    def mapnZnYX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def mapnZnYnX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
         0  0 -1
         0 -1  0
        -1  0  0
        ```

        Returns
        - this
        """
        ...


    def mapnZnYnX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def negateX(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        -1 0 0
         0 1 0
         0 0 1
        ```

        Returns
        - this
        """
        ...


    def negateX(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def negateY(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1  0 0
        0 -1 0
        0  0 1
        ```

        Returns
        - this
        """
        ...


    def negateY(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def negateZ(self) -> "Matrix3f":
        """
        Multiply `this` by the matrix
        ```
        1 0  0
        0 1  0
        0 0 -1
        ```

        Returns
        - this
        """
        ...


    def negateZ(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def clone(self) -> "Object":
        ...



from .FlowSequenceEndToken import FlowSequenceEndToken
from .StreamStartToken import StreamStartToken
from .KeyToken import KeyToken
from .DocumentEndToken import DocumentEndToken
from .FlowSequenceStartToken import FlowSequenceStartToken
from .ScalarToken import ScalarToken
from .Token import Token
from .CommentToken import CommentToken
from .AnchorToken import AnchorToken
from .BlockEndToken import BlockEndToken
from .FlowMappingStartToken import FlowMappingStartToken
from .StreamEndToken import StreamEndToken
from .DirectiveToken import DirectiveToken
from .TagToken import TagToken
from .BlockSequenceStartToken import BlockSequenceStartToken
from .TagTuple import TagTuple
from .FlowEntryToken import FlowEntryToken
from .FlowMappingEndToken import FlowMappingEndToken
from .BlockEntryToken import BlockEntryToken
from .DocumentStartToken import DocumentStartToken
from .ValueToken import ValueToken
from .AliasToken import AliasToken
from .BlockMappingStartToken import BlockMappingStartToken


"""
Python module generated from Java source file org.bukkit.block.ShulkerBox

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import DyeColor
from org.bukkit.block import *
from org.bukkit.loot import Lootable
from typing import Any, Callable, Iterable, Tuple


class ShulkerBox(Container, Lootable, Lidded):
    """
    Represents a captured state of a ShulkerBox.
    """

    def getColor(self) -> "DyeColor":
        """
        Get the DyeColor corresponding to this ShulkerBox

        Returns
        - the DyeColor of this ShulkerBox, or null if default
        """
        ...



"""
Python module generated from Java source file com.google.gson.internal.sql.SqlTypesSupport

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import TypeAdapterFactory
from com.google.gson.internal.bind.DefaultDateTypeAdapter import DateType
from com.google.gson.internal.sql import *
from java.util import Date
from typing import Any, Callable, Iterable, Tuple


class SqlTypesSupport:
    """
    Encapsulates access to `java.sql` types, to allow Gson to work without the `java.sql`
    module being present. No ClassNotFoundExceptions will be thrown in case the `java.sql` module is not present.
    
    If .SUPPORTS_SQL_TYPES is `True`, all other constants of this class will be
    non-`null`. However, if it is `False` all other constants will be `null` and
    there will be no support for `java.sql` types.
    """

    SUPPORTS_SQL_TYPES = None
    """
    `True` if `java.sql` types are supported, `False` otherwise
    """
    DATE_DATE_TYPE = None
    TIMESTAMP_DATE_TYPE = None
    DATE_FACTORY = None
    TIME_FACTORY = None
    TIMESTAMP_FACTORY = None



"""
Python module generated from Java source file com.google.common.collect.ForwardingQueue

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util import NoSuchElementException
from java.util import Queue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingQueue(ForwardingCollection, Queue):
    """
    A queue which forwards all its method calls to another queue. Subclasses should override one or
    more methods to modify the behavior of the backing queue as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    **Warning:** The methods of `ForwardingQueue` forward **indiscriminately** to the
    methods of the delegate. For example, overriding .add alone **will not** change the
    behavior of .offer which can lead to unexpected behavior. In this case, you should
    override `offer` as well, either providing your own implementation, or delegating to the
    provided `standardOffer` method.
    
    **`default` method warning:** This class does *not* forward calls to `default` methods. Instead, it inherits their default implementations. When those implementations
    invoke methods, they invoke methods on the `ForwardingQueue`.
    
    The `standard` methods are not guaranteed to be thread-safe, even when all of the
    methods that they depend on are thread-safe.

    Author(s)
    - Louis Wasserman

    Since
    - 2.0
    """

    def offer(self, o: "E") -> bool:
        ...


    def poll(self) -> "E":
        ...


    def remove(self) -> "E":
        ...


    def peek(self) -> "E":
        ...


    def element(self) -> "E":
        ...



"""
Python module generated from Java source file org.joml.sampling.UniformSampling

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import Random
from org.joml.sampling import *
from typing import Any, Callable, Iterable, Tuple


class UniformSampling:
    """
    Generates uniform samples.

    Author(s)
    - Kai Burjack
    """

    class Disk:
        """
        Generates uniform samples on a unit disk.

    Author(s)
        - Kai Burjack
        """

        def __init__(self, seed: int, numSamples: int, callback: "Callback2d"):
            """
            Create a new instance of Disk, initialize the random number generator with the given `seed` and generate `numSamples` number of sample
            positions on the unit disk, and call the given `callback` for each sample generate.

            Arguments
            - seed: the seed to initialize the random number generator with
            - numSamples: the number of samples to generate
            - callback: will be called for each sample generated
            """
            ...


    class Sphere:
        """
        Generates uniform samples on a unit sphere.

    Author(s)
        - Kai Burjack
        """

        def __init__(self, seed: int, numSamples: int, callback: "Callback3d"):
            """
            Create a new instance of Sphere, initialize the random number generator with the given `seed` and generate `numSamples` number of sample
            positions on the unit sphere, and call the given `callback` for each sample generate.

            Arguments
            - seed: the seed to initialize the random number generator with
            - numSamples: the number of samples to generate
            - callback: will be called for each sample generated
            """
            ...


        def generate(self, numSamples: int, callback: "Callback3d") -> None:
            """
            Create `numSamples` number of samples which are uniformly distributed on a unit sphere, and call the given `callback` for each sample generated.
            
            Reference: <a href="http://mathworld.wolfram.com/SpherePointPicking.html">http://mathworld.wolfram.com/</a>

            Arguments
            - numSamples: the number of samples to generate
            - callback: will be called for each sample generated
            """
            ...



"""
Python module generated from Java source file com.google.common.io.AppendableWriter

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from java.io import Closeable
from java.io import Flushable
from java.io import IOException
from java.io import Writer
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class AppendableWriter(Writer):
    """
    Writer that places all output on an Appendable target. If the target is Flushable
    or Closeable, flush()es and close()s will also be delegated to the target.

    Author(s)
    - Sebastian Kanthak

    Since
    - 1.0
    """

    def write(self, cbuf: list[str], off: int, len: int) -> None:
        ...


    def write(self, c: int) -> None:
        ...


    def write(self, str: str) -> None:
        ...


    def write(self, str: str, off: int, len: int) -> None:
        ...


    def flush(self) -> None:
        ...


    def close(self) -> None:
        ...


    def append(self, c: str) -> "Writer":
        ...


    def append(self, charSeq: "CharSequence") -> "Writer":
        ...


    def append(self, charSeq: "CharSequence", start: int, end: int) -> "Writer":
        ...



"""
Python module generated from Java source file java.lang.ref.Reference

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.ref import *
from jdk.internal.access import JavaLangRefAccess
from jdk.internal.access import SharedSecrets
from jdk.internal.ref import Cleaner
from jdk.internal.vm.annotation import ForceInline
from jdk.internal.vm.annotation import IntrinsicCandidate
from typing import Any, Callable, Iterable, Tuple


class Reference:

    def get(self) -> "T":
        """
        Returns this reference object's referent.  If this reference object has
        been cleared, either by the program or by the garbage collector, then
        this method returns `null`.

        Returns
        - The object to which this reference refers, or
                  `null` if this reference object has been cleared

        See
        - .refersTo

        Unknown Tags
        - This method returns a strong reference to the referent. This may cause
        the garbage collector to treat it as strongly reachable until some later
        collection cycle.  The .refersTo(Object) refersTo method can be
        used to avoid such strengthening when testing whether some object is
        the referent of a reference object; that is, use `ref.refersTo(obj)`
        rather than `ref.get() == obj`.
        """
        ...


    def refersTo(self, obj: "T") -> bool:
        """
        Tests if the referent of this reference object is `obj`.
        Using a `null` `obj` returns `True` if the
        reference object has been cleared.

        Arguments
        - obj: the object to compare with this reference object's referent

        Returns
        - `True` if `obj` is the referent of this reference object

        Since
        - 16
        """
        ...


    def clear(self) -> None:
        """
        Clears this reference object.  Invoking this method will not cause this
        object to be enqueued.
        
         This method is invoked only by Java code; when the garbage collector
        clears references it does so directly, without invoking this method.
        """
        ...


    def isEnqueued(self) -> bool:
        """
        Tests if this reference object is in its associated queue, if any.
        This method returns `True` only if all of the following conditions
        are met:
        
        - this reference object was registered with a queue when it was created; and
        - the garbage collector has added this reference object to the queue
            or .enqueue() is called; and
        - this reference object is not yet removed from the queue.
        
        Otherwise, this method returns `False`.
        This method may return `False` if this reference object has been cleared
        but not enqueued due to the race condition.

        Returns
        - `True` if and only if this reference object is
                  in its associated queue (if any).

        Deprecated
        - This method was originally specified to test if a reference object has
        been cleared and enqueued but was never implemented to do this test.
        This method could be misused due to the inherent race condition
        or without an associated `ReferenceQueue`.
        An application relying on this method to release critical resources
        could cause serious performance issue.
        An application should use ReferenceQueue to reliably determine
        what reference objects that have been enqueued or
        .refersTo(Object) refersTo(null) to determine if this reference
        object has been cleared.
        """
        ...


    def enqueue(self) -> bool:
        """
        Clears this reference object and adds it to the queue with which
        it is registered, if any.
        
         This method is invoked only by Java code; when the garbage collector
        enqueues references it does so directly, without invoking this method.

        Returns
        - `True` if this reference object was successfully
                  enqueued; `False` if it was already enqueued or if
                  it was not registered with a queue when it was created
        """
        ...


    @staticmethod
    def reachabilityFence(ref: "Object") -> None:
        """
        Ensures that the object referenced by the given reference remains
        <a href="package-summary.html#reachability">*strongly reachable*</a>,
        regardless of any prior actions of the program that might otherwise cause
        the object to become unreachable; thus, the referenced object is not
        reclaimable by garbage collection at least until after the invocation of
        this method.  Invocation of this method does not itself initiate garbage
        collection or finalization.
        
         This method establishes an ordering for
        <a href="package-summary.html#reachability">*strong reachability*</a>
        with respect to garbage collection.  It controls relations that are
        otherwise only implicit in a program -- the reachability conditions
        triggering garbage collection.  This method is designed for use in
        uncommon situations of premature finalization where using
        `synchronized` blocks or methods, or using other synchronization
        facilities are not possible or do not provide the desired control.  This
        method is applicable only when reclamation may have visible effects,
        which is possible for objects with finalizers (See Section 12.6
        of <cite>The Java Language Specification</cite>) that
        are implemented in ways that rely on ordering control for
        correctness.

        Arguments
        - ref: the reference. If `null`, this method has no effect.

        Since
        - 9

        Unknown Tags
        - Finalization may occur whenever the virtual machine detects that no
        reference to an object will ever be stored in the heap: The garbage
        collector may reclaim an object even if the fields of that object are
        still in use, so long as the object has otherwise become unreachable.
        This may have surprising and undesirable effects in cases such as the
        following example in which the bookkeeping associated with a class is
        managed through array indices.  Here, method `action` uses a
        `reachabilityFence` to ensure that the `Resource` object is
        not reclaimed before bookkeeping on an associated
        `ExternalResource` has been performed; in particular here, to
        ensure that the array slot holding the `ExternalResource` is not
        nulled out in method Object.finalize, which may otherwise run
        concurrently.
        
        ``` `class Resource {
          private static ExternalResource[] externalResourceArray = ...
        
          int myIndex;
          Resource(...) {
            myIndex = ...
            externalResourceArray[myIndex] = ...;
            ...`
          protected void finalize() {
            externalResourceArray[myIndex] = null;
            ...
          }
          public void action() {
            try {
              // ...
              int i = myIndex;
              Resource.update(externalResourceArray[i]);
            } finally {
              Reference.reachabilityFence(this);
            }
          }
          private static void update(ExternalResource ext) {
            ext.status = ...;
          }
        }}```
        
        Here, the invocation of `reachabilityFence` is nonintuitively
        placed *after* the call to `update`, to ensure that the
        array slot is not nulled out by Object.finalize before the
        update, even if the call to `action` was the last use of this
        object.  This might be the case if, for example a usage in a user program
        had the form `new Resource().action();` which retains no other
        reference to this `Resource`.  While probably overkill here,
        `reachabilityFence` is placed in a `finally` block to ensure
        that it is invoked across all paths in the method.  In a method with more
        complex control paths, you might need further precautions to ensure that
        `reachabilityFence` is encountered along all of them.
        
         It is sometimes possible to better encapsulate use of
        `reachabilityFence`.  Continuing the above example, if it were
        acceptable for the call to method `update` to proceed even if the
        finalizer had already executed (nulling out slot), then you could
        localize use of `reachabilityFence`:
        
        ``` `public void action2() {
          // ...
          Resource.update(getExternalResource());`
        private ExternalResource getExternalResource() {
          ExternalResource ext = externalResourceArray[myIndex];
          Reference.reachabilityFence(this);
          return ext;
        }}```
        
         Method `reachabilityFence` is not required in constructions
        that themselves ensure reachability.  For example, because objects that
        are locked cannot, in general, be reclaimed, it would suffice if all
        accesses of the object, in all methods of class `Resource`
        (including `finalize`) were enclosed in `synchronized (this)`
        blocks.  (Further, such blocks must not include infinite loops, or
        themselves be unreachable, which fall into the corner case exceptions to
        the "in general" disclaimer.)  However, method `reachabilityFence`
        remains a better option in cases where this approach is not as efficient,
        desirable, or possible; for example because it would encounter deadlock.
        - 12.6 Finalization of Class Instances
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AbstractService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.common.util.concurrent.Monitor import Guard
from com.google.common.util.concurrent.Service import State
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import ForOverride
from com.google.errorprone.annotations.concurrent import GuardedBy
from com.google.j2objc.annotations import WeakOuter
from java.time import Duration
from java.util.concurrent import Executor
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class AbstractService(Service):
    """
    Base class for implementing services that can handle .doStart and .doStop
    requests, responding to them with .notifyStarted() and .notifyStopped()
    callbacks. Its subclasses must manage threads manually; consider AbstractExecutionThreadService if you need only a single execution thread.

    Author(s)
    - Luke Sandberg

    Since
    - 1.0
    """

    def startAsync(self) -> "Service":
        ...


    def stopAsync(self) -> "Service":
        ...


    def awaitRunning(self) -> None:
        ...


    def awaitRunning(self, timeout: "Duration") -> None:
        """
        Since
        - 28.0
        """
        ...


    def awaitRunning(self, timeout: int, unit: "TimeUnit") -> None:
        ...


    def awaitTerminated(self) -> None:
        ...


    def awaitTerminated(self, timeout: "Duration") -> None:
        """
        Since
        - 28.0
        """
        ...


    def awaitTerminated(self, timeout: int, unit: "TimeUnit") -> None:
        ...


    def isRunning(self) -> bool:
        ...


    def state(self) -> "State":
        ...


    def failureCause(self) -> "Throwable":
        """
        Since
        - 14.0
        """
        ...


    def addListener(self, listener: "Listener", executor: "Executor") -> None:
        """
        Since
        - 13.0
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.configuration.file.YamlConfiguration

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.io import ByteArrayInputStream
from java.io import File
from java.io import FileNotFoundException
from java.io import IOException
from java.io import Reader
from java.io import StringWriter
from java.nio.charset import StandardCharsets
from org.bukkit import Bukkit
from org.bukkit.configuration import Configuration
from org.bukkit.configuration import ConfigurationSection
from org.bukkit.configuration import InvalidConfigurationException
from org.bukkit.configuration.file import *
from org.bukkit.configuration.serialization import ConfigurationSerialization
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml import Yaml
from org.yaml.snakeyaml.comments import CommentLine
from org.yaml.snakeyaml.comments import CommentType
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.nodes import AnchorNode
from org.yaml.snakeyaml.nodes import MappingNode
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import NodeTuple
from org.yaml.snakeyaml.nodes import ScalarNode
from org.yaml.snakeyaml.nodes import SequenceNode
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.reader import UnicodeReader
from typing import Any, Callable, Iterable, Tuple


class YamlConfiguration(FileConfiguration):
    """
    An implementation of Configuration which saves all files in Yaml.
    Note that this implementation is not synchronized.
    """

    def __init__(self):
        ...


    def saveToString(self) -> str:
        ...


    def loadFromString(self, contents: str) -> None:
        ...


    def options(self) -> "YamlConfigurationOptions":
        ...


    @staticmethod
    def loadConfiguration(file: "File") -> "YamlConfiguration":
        """
        Creates a new YamlConfiguration, loading from the given file.
        
        Any errors loading the Configuration will be logged and then ignored.
        If the specified input is not a valid config, a blank config will be
        returned.
        
        The encoding used may follow the system dependent default.

        Arguments
        - file: Input file

        Returns
        - Resulting configuration

        Raises
        - IllegalArgumentException: Thrown if file is null
        """
        ...


    @staticmethod
    def loadConfiguration(reader: "Reader") -> "YamlConfiguration":
        """
        Creates a new YamlConfiguration, loading from the given reader.
        
        Any errors loading the Configuration will be logged and then ignored.
        If the specified input is not a valid config, a blank config will be
        returned.

        Arguments
        - reader: input

        Returns
        - resulting configuration

        Raises
        - IllegalArgumentException: Thrown if stream is null
        """
        ...



"""
Python module generated from Java source file org.bukkit.Registry

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.base import Predicates
from com.google.common.collect import ImmutableMap
from java.util import Iterator
from java.util import Locale
from java.util import Objects
from java.util.function import Predicate
from java.util.stream import Stream
from java.util.stream import StreamSupport
from org.bukkit import *
from org.bukkit.advancement import Advancement
from org.bukkit.attribute import Attribute
from org.bukkit.block import Biome
from org.bukkit.block import BlockType
from org.bukkit.block.banner import PatternType
from org.bukkit.boss import KeyedBossBar
from org.bukkit.damage import DamageType
from org.bukkit.enchantments import Enchantment
from org.bukkit.entity import Cat
from org.bukkit.entity import EntityType
from org.bukkit.entity import Frog
from org.bukkit.entity import Villager
from org.bukkit.entity import Wolf
from org.bukkit.entity.memory import MemoryKey
from org.bukkit.generator.structure import Structure
from org.bukkit.generator.structure import StructureType
from org.bukkit.inventory import ItemType
from org.bukkit.inventory import MenuType
from org.bukkit.inventory.meta.trim import TrimMaterial
from org.bukkit.inventory.meta.trim import TrimPattern
from org.bukkit.loot import LootTable
from org.bukkit.loot import LootTables
from org.bukkit.map import MapCursor
from org.bukkit.potion import PotionEffectType
from org.bukkit.potion import PotionType
from typing import Any, Callable, Iterable, Tuple


class Registry(Iterable):
    """
    Represents a registry of Bukkit objects that may be retrieved by
    NamespacedKey.
    
    Type `<T>`: type of item in the registry
    """

    ADVANCEMENT = Registry<Advancement>() {
    
        @Nullable
        @Override
        public Advancement get(@NotNull NamespacedKey key) {
            return Bukkit.getAdvancement(key);
        }
    
        @NotNull
        @Override
        public Advancement getOrThrow(@NotNull NamespacedKey key) {
            Advancement advancement = get(key);
            Preconditions.checkArgument(advancement != null, "No Advancement registry entry found for key %s.", key);
            return advancement;
        }
    
        @NotNull
        @Override
        public Stream<Advancement> stream() {
            return StreamSupport.stream(spliterator(), false);
        }
    
        @NotNull
        @Override
        public Iterator<Advancement> iterator() {
            return Bukkit.advancementIterator();
        }
    }
    """
    Server advancements.

    See
    - Bukkit.advancementIterator()

    Deprecated
    - Advancement has no real server-side registry.
    """
    ART = Objects.requireNonNull(Bukkit.getRegistry(Art.class), "No registry present for Art. This is a bug.")
    """
    Server art.

    See
    - Art
    """
    ATTRIBUTE = Objects.requireNonNull(Bukkit.getRegistry(Attribute.class), "No registry present for Attribute. This is a bug.")
    """
    Attribute.

    See
    - Attribute
    """
    BANNER_PATTERN = Objects.requireNonNull(Bukkit.getRegistry(PatternType.class), "No registry present for Pattern Type. This is a bug.")
    """
    Server banner patterns.

    See
    - PatternType
    """
    BIOME = Objects.requireNonNull(Bukkit.getRegistry(Biome.class), "No registry present for Biome. This is a bug.")
    """
    Server biomes.

    See
    - Biome
    """
    BLOCK = Objects.requireNonNull(Bukkit.getRegistry(BlockType.class), "No registry present for BlockType. This is a bug.")
    """
    Server block types.

    See
    - BlockType

    Unknown Tags
    - BlockType is not ready for public usage yet
    """
    BOSS_BARS = Registry<KeyedBossBar>() {
    
        @Nullable
        @Override
        public KeyedBossBar get(@NotNull NamespacedKey key) {
            return Bukkit.getBossBar(key);
        }
    
        @NotNull
        @Override
        public KeyedBossBar getOrThrow(@NotNull NamespacedKey key) {
            KeyedBossBar keyedBossBar = get(key);
            Preconditions.checkArgument(keyedBossBar != null, "No KeyedBossBar registry entry found for key %s.", key);
            return keyedBossBar;
        }
    
        @NotNull
        @Override
        public Stream<KeyedBossBar> stream() {
            return StreamSupport.stream(spliterator(), false);
        }
    
        @NotNull
        @Override
        public Iterator<KeyedBossBar> iterator() {
            return Bukkit.getBossBars();
        }
    }
    """
    Custom boss bars.

    See
    - Bukkit.getBossBars()

    Deprecated
    - BossBar has no real server-side registry.
    """
    CAT_VARIANT = Objects.requireNonNull(Bukkit.getRegistry(Cat.Type.class), "No registry present for Cat Type. This is a bug.")
    """
    Server cat types.

    See
    - Cat.Type
    """
    ENCHANTMENT = Objects.requireNonNull(Bukkit.getRegistry(Enchantment.class), "No registry present for Enchantment. This is a bug.")
    """
    Server enchantments.

    See
    - Enchantment
    """
    ENTITY_TYPE = SimpleRegistry<>(EntityType.class, (entity) -> entity != EntityType.UNKNOWN)
    """
    Server entity types.

    See
    - EntityType
    """
    INSTRUMENT = Objects.requireNonNull(Bukkit.getRegistry(MusicInstrument.class), "No registry present for MusicInstrument. This is a bug.")
    """
    Server instruments.

    See
    - MusicInstrument
    """
    ITEM = Objects.requireNonNull(Bukkit.getRegistry(ItemType.class), "No registry present for ItemType. This is a bug.")
    """
    Server item types.

    See
    - ItemType

    Unknown Tags
    - ItemType is not ready for public usage yet
    """
    LOOT_TABLES = SimpleRegistry<>(LootTables.class)
    """
    Default server loot tables.

    See
    - LootTables

    Deprecated
    - LootTables, not to be confused with the LootTable class, has no real server-side registry.
    """
    MATERIAL = SimpleRegistry<>(Material.class, (mat) -> !mat.isLegacy())
    """
    Server materials.

    See
    - Material
    """
    MENU = Objects.requireNonNull(Bukkit.getRegistry(MenuType.class), "No registry present for MenuType. This is a bug.")
    """
    Server menus.

    See
    - MenuType
    """
    EFFECT = Objects.requireNonNull(Bukkit.getRegistry(PotionEffectType.class), "No registry present for PotionEffectType. This is a bug.")
    """
    Server mob effects.

    See
    - PotionEffectType
    """
    PARTICLE_TYPE = SimpleRegistry<>(Particle.class, (par) -> par.register)
    """
    Server particles.

    See
    - Particle
    """
    POTION = SimpleRegistry<>(PotionType.class)
    """
    Server potions.

    See
    - PotionType
    """
    STATISTIC = SimpleRegistry<>(Statistic.class)
    """
    Server statistics.

    See
    - Statistic
    """
    STRUCTURE = Objects.requireNonNull(Bukkit.getRegistry(Structure.class), "No registry present for Structure. This is a bug.")
    """
    Server structures.

    See
    - Structure
    """
    STRUCTURE_TYPE = Objects.requireNonNull(Bukkit.getRegistry(StructureType.class), "No registry present for StructureType. This is a bug.")
    """
    Server structure types.

    See
    - StructureType
    """
    SOUNDS = Objects.requireNonNull(Bukkit.getRegistry(Sound.class), "No registry present for Sound. This is a bug.")
    """
    Sound keys.

    See
    - Sound
    """
    TRIM_MATERIAL = Objects.requireNonNull(Bukkit.getRegistry(TrimMaterial.class), "No registry present for TrimMaterial. This is a bug.")
    """
    Trim materials.

    See
    - TrimMaterial
    """
    TRIM_PATTERN = Objects.requireNonNull(Bukkit.getRegistry(TrimPattern.class), "No registry present for TrimPattern. This is a bug.")
    """
    Trim patterns.

    See
    - TrimPattern
    """
    DAMAGE_TYPE = Objects.requireNonNull(Bukkit.getRegistry(DamageType.class), "No registry present for DamageType. This is a bug.")
    """
    Damage types.

    See
    - DamageType
    """
    JUKEBOX_SONG = Objects.requireNonNull(Bukkit.getRegistry(JukeboxSong.class), "No registry present for JukeboxSong. This is a bug.")
    """
    Jukebox songs.

    See
    - JukeboxSong
    """
    VILLAGER_PROFESSION = Objects.requireNonNull(Bukkit.getRegistry(Villager.Profession.class), "No registry present for Villager Profession. This is a bug.")
    """
    Villager profession.

    See
    - Villager.Profession
    """
    VILLAGER_TYPE = Objects.requireNonNull(Bukkit.getRegistry(Villager.Type.class), "No registry present for Villager Type. This is a bug.")
    """
    Villager type.

    See
    - Villager.Type
    """
    MEMORY_MODULE_TYPE = Registry<MemoryKey>() {
    
        @NotNull
        @Override
        public Iterator iterator() {
            return MemoryKey.values().iterator();
        }
    
        @Nullable
        @Override
        public MemoryKey get(@NotNull NamespacedKey key) {
            return MemoryKey.getByKey(key);
        }
    
        @NotNull
        @Override
        public MemoryKey getOrThrow(@NotNull NamespacedKey key) {
            MemoryKey memoryKey = get(key);
            Preconditions.checkArgument(memoryKey != null, "No MemoryKey registry entry found for key %s.", key);
            return memoryKey;
        }
    
        @NotNull
        @Override
        public Stream<MemoryKey> stream() {
            return StreamSupport.stream(spliterator(), false);
        }
    }
    """
    Memory Keys.

    See
    - MemoryKey
    """
    FLUID = Objects.requireNonNull(Bukkit.getRegistry(Fluid.class), "No registry present for Fluid. This is a bug.")
    """
    Server fluids.

    See
    - Fluid
    """
    FROG_VARIANT = Objects.requireNonNull(Bukkit.getRegistry(Frog.Variant.class), "No registry present for Frog Variant. This is a bug.")
    """
    Frog variants.

    See
    - Frog.Variant
    """
    WOLF_VARIANT = Objects.requireNonNull(Bukkit.getRegistry(Wolf.Variant.class), "No registry present for Wolf Variant. This is a bug.")
    """
    Wolf variants.

    See
    - Wolf.Variant
    """
    MAP_DECORATION_TYPE = Objects.requireNonNull(Bukkit.getRegistry(MapCursor.Type.class), "No registry present for MapCursor Type. This is a bug.")
    """
    Map cursor types.

    See
    - MapCursor.Type
    """
    GAME_EVENT = Objects.requireNonNull(Bukkit.getRegistry(GameEvent.class), "No registry present for GameEvent. This is a bug.")
    """
    Game events.

    See
    - GameEvent
    """


    def get(self, key: "NamespacedKey") -> "T":
        """
        Get the object by its key.

        Arguments
        - key: non-null key

        Returns
        - item or null if does not exist
        """
        ...


    def getOrThrow(self, key: "NamespacedKey") -> "T":
        """
        Get the object by its key.
        
        If there is no object with the given key, an exception will be thrown.

        Arguments
        - key: to get the object from

        Returns
        - object with the given key

        Raises
        - IllegalArgumentException: if there is no object with the given key
        """
        ...


    def stream(self) -> "Stream"["T"]:
        """
        Returns a new stream, which contains all registry items, which are registered to the registry.

        Returns
        - a stream of all registry items
        """
        ...


    def match(self, input: str) -> "T":
        """
        Attempts to match the registered object with the given key.
        
        This will attempt to find a reasonable match based on the provided input
        and may do so through unspecified means.

        Arguments
        - input: non-null input

        Returns
        - registered object or null if does not exist
        """
        ...



"""
Python module generated from Java source file org.bukkit.MusicInstrument

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Lists
from java.util import Collections
from org.bukkit import *
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class MusicInstrument(Keyed, RegistryAware):

    PONDER_GOAT_HORN = getInstrument("ponder_goat_horn")
    SING_GOAT_HORN = getInstrument("sing_goat_horn")
    SEEK_GOAT_HORN = getInstrument("seek_goat_horn")
    FEEL_GOAT_HORN = getInstrument("feel_goat_horn")
    ADMIRE_GOAT_HORN = getInstrument("admire_goat_horn")
    CALL_GOAT_HORN = getInstrument("call_goat_horn")
    YEARN_GOAT_HORN = getInstrument("yearn_goat_horn")
    DREAM_GOAT_HORN = getInstrument("dream_goat_horn")


    def getDuration(self) -> float:
        """
        Gets how long the use duration is for the instrument.

        Returns
        - the duration.
        """
        ...


    def getRange(self) -> float:
        """
        Gets the range of the sound.

        Returns
        - the range of the sound.
        """
        ...


    def getDescription(self) -> str:
        """
        Gets the description of this instrument.

        Returns
        - the description.
        """
        ...


    def getSoundEvent(self) -> "Sound":
        """
        Gets the sound/sound-event for this instrument.

        Returns
        - a sound.
        """
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    @staticmethod
    def getByKey(namespacedKey: "NamespacedKey") -> "MusicInstrument":
        """
        Returns a MusicInstrument by a NamespacedKey.

        Arguments
        - namespacedKey: the key

        Returns
        - the event or null

        Deprecated
        - Use Registry.get(NamespacedKey) instead.
        """
        ...


    @staticmethod
    def values() -> Iterable["MusicInstrument"]:
        """
        Returns all known MusicInstruments.

        Returns
        - the memoryKeys

        Deprecated
        - use Registry.iterator().
        """
        ...
